// Headerfile for loading executables
// UXE = Ulsop eXEcutable

/*
Basic UXE-Concept (Header):         Bit     Content
    Byte    Size   Content
      0-1   WORD    "SL"
      2-3   WORD    Flags:
                                     0-3        Reserved (0,0,0,0)
                                     4          App uses screen
                                     5          Stack (std.val = 32 (1/2 KB))
                                     6          Set size (std.val = progsize)
                                     7-15       Reserved
    ? 4-5   WORD   Only if flag 5 is set: Stacksize in Paragraphs
    ?      8 Byte  Only if flag 4 is set: Short prog.identifier
    ?       WORD   Only if flag 6 is set: Bytes to allocate

    ?         ?    Code

The organisation is always 0h (No PSP)
*/

// Overflow Direction Interrupt Trap Sign Zero Auxilary Parity Carry
// 0000ODIT SZ0A0P1C
// 00000010 00000010
#define DEFFLAGS 0x7202

UINT getsomemem(UINT howmuch)
    {
    UINT i;
    _asm
        {
        mov ah, 48h
        mov bx, howmuch
        int 21h
        jc reserr
        mov i, ax
        jmp noreserr
reserr: mov ax, 0
        mov i, ax
noreserr:
        }
    return i;
    }

char runuxe(char *filename)
    {
    FILE *fp=fopen(filename,"r+b");
    UINT flags;
    char dat[2];
    UCHR *appcs;
    UINT hmstack=64, i;
    UINT fsize, para, appsstack;
    char *appident=NULL;

    if(fp == NULL)
        return 1;       // Cannot read

    fseek(fp,0,SEEK_END);
    fsize = ftell(fp) - 3;
    fseek(fp,0,SEEK_SET);

    fread(dat,1,2,fp);
    if(!(dat[0] == 'S' && dat[1] == 'L'))
        {
        fclose(fp);     
        return 2;       // Invalid file
        }
    fread(&flags,1,2,fp);
    if(flags & 32)
        fsize-=2, fread(&hmstack,1,2,fp);           // Specified Stacksize
    if(flags & 16)
        {
        appident = malloc(9);                       // Screen
        fread(appident,1,8,fp);
        appident[8] = 0;
        fsize -= 8;
        }
    if(flags & 64)
        fread(&fsize,1,2,fp);                       // Bytesalloc
    // Header completed

    if((appsstack = getsomemem(hmstack)) == 0   )   // Reserve stack-space
        return 3;       // Out of memory

    para = (fsize + 15) / 16;                       // We need so many paras.
    para = getsomemem(para);
    if(para == 0)
        {                                           // Again: Out of mem.
        _asm
            {
            mov ax, appsstack;
            mov es, ax;
            mov ah, 49h
            int 21h
            }
        return 4;
        }
    // Load the proggie
    fread(MK_FP(para,0),fsize,1,fp);
    fclose(fp);

    // And add it to tasktable
    for(i=0;i<64;i++)
        if(tasktable[i].status == -1)
            {
            tasktable[i].status = 0;
            tasktable[i].taskname = appident;
            tasktable[i].lastsig = 1;               // Paint. Signal
            tasktable[i].ss = appsstack;
            tasktable[i].sp = (hmstack * 16) - 2;

            *((UINT *) MK_FP(appsstack,tasktable[i].sp)) = DEFFLAGS;     // Flags
            tasktable[i].sp -= 2;
            *((UINT *) MK_FP(appsstack,tasktable[i].sp)) = para;         // CS
            tasktable[i].sp -= 2;
            *((UINT *) MK_FP(appsstack,tasktable[i].sp)) = 0;            // IP

                // We've got it! Task is running
            fgtask = i;     // Don't forget to set it to fg
            if(curdispx != 0)
                drawthetaskbar();
            return 0;
            }

    // Oh, no! No space in tasktable!
    // Nothing we can do, remove it
    _asm
        {
        mov ax, appsstack;
        mov es, ax;
        mov ah, 49h
        int 21h
        mov ax, para;
        mov es, ax;
        mov ah, 49h
        int 21h
        }
    return 5;
    }
